@@ -132,6 +132,12 @@ def parse_args():
         action='store_true',
         help='To be used when the worker should only run bundles that match the worker\'s tag.',
     )
+    parser.add_argument(
+        '--terminate',
+        action='store_true',
+        help='Terminate the worker and kill all the existing running bundles.',
+    )
+
     return parser.parse_args()
 
 
@@ -222,6 +228,7 @@ def main():
         args.tag_exclusive,
         docker_runtime=docker_runtime,
         docker_network_prefix=args.network_prefix,
+        terminate=args.terminate,
     )
 
     # Register a signal handler to ensure safe shutdown.
